import asyncio
import logging
from datetime import datetime, timedelta, timezone

import aiohttp
from utils import create_or_get_sandbox

from blaxel.client.models import Metadata, Preview, PreviewSpec
from blaxel.common.settings import settings
from blaxel.sandbox.sandbox import SandboxInstance

logger = logging.getLogger(__name__)

async def test_public_preview(sandbox: SandboxInstance):
    try:
        # Create a public preview
        await sandbox.previews.create(Preview(
            metadata=Metadata(name="preview-test-public"),
            spec=PreviewSpec(
                port=443,
                prefix_url="small-prefix",
                public=True
            )
        ))

        # List previews
        previews = await sandbox.previews.list()
        assert len(previews) >= 1, "No previews found"

        # Get the preview
        retrieved_preview = await sandbox.previews.get("preview-test-public")
        assert retrieved_preview.name == "preview-test-public", "Preview name is not correct"

        # Check the URL
        url = retrieved_preview.spec.url if retrieved_preview.spec else None
        assert url is not None, "Preview URL is not correct"
        workspace = settings.workspace
        expected_url = f"https://small-prefix-{workspace}.preview.bl.run"
        assert url == expected_url, f"Preview URL is not correct => {url}"

        # Test the preview endpoint
        async with aiohttp.ClientSession() as session:
            async with session.get(f"{url}/health") as response:
                assert response.status == 200, f"Preview is not working => {response.status}:{await response.text()}"

        logger.info("Public preview is healthy :)")
    except Exception as e:
        logger.error("ERROR IN PUBLIC PREVIEW TEST => ", exc_info=e)
        raise
    finally:
        await sandbox.previews.delete("preview-test-public")

async def test_private_preview(sandbox: SandboxInstance):
    try:
        # Create a private preview
        preview = await sandbox.previews.create(Preview(
            metadata=Metadata(name="preview-test-private"),
            spec=PreviewSpec(
                port=443,
                public=False
            )
        ))

        # Get the preview URL
        url = preview.spec.url if preview.spec else None
        assert url is not None, "Preview URL is not correct"

        # Create a token
        token = await preview.tokens.create(datetime.now(timezone.utc) + timedelta(minutes=10))
        logger.info(f"Token created => {token.value}")

        # List tokens
        tokens = await preview.tokens.list()
        assert len(tokens) >= 1, "No tokens found"
        assert any(t.value == token.value for t in tokens), "Token not found in list"

        # Test the preview endpoint without token
        async with aiohttp.ClientSession() as session:
            async with session.get(f"{url}/health") as response:
                assert response.status == 401, f"Preview is not protected by token, response => {response.status}"

        # Test the preview endpoint with token
        async with aiohttp.ClientSession() as session:
            async with session.get(f"{url}/health?bl_preview_token={token.value}") as response:
                assert response.status == 200, f"Preview is not working with token, response => {response.status}"

        logger.info("Private preview is healthy with token :)")

        # Delete the token
        await preview.tokens.delete(token.value)
    except Exception as e:
        logger.error("ERROR IN PRIVATE PREVIEW TEST => ", exc_info=e)
        raise
    finally:
        await sandbox.previews.delete("preview-test-private")

async def main():
    sandbox_name = "sandbox-preview-test"
    sandbox = None
    try:
        sandbox = await create_or_get_sandbox(sandbox_name)
        await test_public_preview(sandbox)
        await test_private_preview(sandbox)
    finally:
        if sandbox:
            logger.info("Deleting sandbox")
            await SandboxInstance.delete(sandbox_name)

if __name__ == "__main__":
    asyncio.run(main())


from typing import Any, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.process_response_status import ProcessResponseStatus
from ..types import UNSET, Unset

T = TypeVar("T", bound="ProcessResponse")


@_attrs_define
class ProcessResponse:
    """
    Attributes:
        command (Union[Unset, str]):  Example: ls -la.
        completed_at (Union[Unset, str]):  Example: Wed, 01 Jan 2023 12:01:00 GMT.
        exit_code (Union[Unset, int]):
        name (Union[Unset, str]):  Example: my-process.
        pid (Union[Unset, str]):  Example: 1234.
        started_at (Union[Unset, str]):  Example: Wed, 01 Jan 2023 12:00:00 GMT.
        status (Union[Unset, ProcessResponseStatus]):  Example: running.
        working_dir (Union[Unset, str]):  Example: /home/user.
    """

    command: Union[Unset, str] = UNSET
    completed_at: Union[Unset, str] = UNSET
    exit_code: Union[Unset, int] = UNSET
    name: Union[Unset, str] = UNSET
    pid: Union[Unset, str] = UNSET
    started_at: Union[Unset, str] = UNSET
    status: Union[Unset, ProcessResponseStatus] = UNSET
    working_dir: Union[Unset, str] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        command = self.command

        completed_at = self.completed_at

        exit_code = self.exit_code

        name = self.name

        pid = self.pid

        started_at = self.started_at

        status: Union[Unset, str] = UNSET
        if not isinstance(self.status, Unset):
            status = self.status.value

        working_dir = self.working_dir

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if command is not UNSET:
            field_dict["command"] = command
        if completed_at is not UNSET:
            field_dict["completedAt"] = completed_at
        if exit_code is not UNSET:
            field_dict["exitCode"] = exit_code
        if name is not UNSET:
            field_dict["name"] = name
        if pid is not UNSET:
            field_dict["pid"] = pid
        if started_at is not UNSET:
            field_dict["startedAt"] = started_at
        if status is not UNSET:
            field_dict["status"] = status
        if working_dir is not UNSET:
            field_dict["workingDir"] = working_dir

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        if not src_dict:
            return None
        d = src_dict.copy()
        command = d.pop("command", UNSET)

        completed_at = d.pop("completedAt", UNSET)

        exit_code = d.pop("exitCode", UNSET)

        name = d.pop("name", UNSET)

        pid = d.pop("pid", UNSET)

        started_at = d.pop("startedAt", UNSET)

        _status = d.pop("status", UNSET)
        status: Union[Unset, ProcessResponseStatus]
        if isinstance(_status, Unset):
            status = UNSET
        else:
            status = ProcessResponseStatus(_status)

        working_dir = d.pop("workingDir", UNSET)

        process_response = cls(
            command=command,
            completed_at=completed_at,
            exit_code=exit_code,
            name=name,
            pid=pid,
            started_at=started_at,
            status=status,
            working_dir=working_dir,
        )

        process_response.additional_properties = d
        return process_response

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="ProcessLogs")


@_attrs_define
class ProcessLogs:
    """
    Attributes:
        logs (Union[Unset, str]):  Example: logs output.
        stderr (Union[Unset, str]):  Example: stderr output.
        stdout (Union[Unset, str]):  Example: stdout output.
    """

    logs: Union[Unset, str] = UNSET
    stderr: Union[Unset, str] = UNSET
    stdout: Union[Unset, str] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        logs = self.logs

        stderr = self.stderr

        stdout = self.stdout

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if logs is not UNSET:
            field_dict["logs"] = logs
        if stderr is not UNSET:
            field_dict["stderr"] = stderr
        if stdout is not UNSET:
            field_dict["stdout"] = stdout

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        if not src_dict:
            return None
        d = src_dict.copy()
        logs = d.pop("logs", UNSET)

        stderr = d.pop("stderr", UNSET)

        stdout = d.pop("stdout", UNSET)

        process_logs = cls(
            logs=logs,
            stderr=stderr,
            stdout=stdout,
        )

        process_logs.additional_properties = d
        return process_logs

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="PreviewTokenSpec")


@_attrs_define
class PreviewTokenSpec:
    """Spec for a Preview Token

    Attributes:
        expired (Union[Unset, bool]): Whether the token is expired
        expires_at (Union[Unset, str]): Expiration time of the token
        token (Union[Unset, str]): Token
    """

    expired: Union[Unset, bool] = UNSET
    expires_at: Union[Unset, str] = UNSET
    token: Union[Unset, str] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        expired = self.expired

        expires_at = self.expires_at

        token = self.token

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if expired is not UNSET:
            field_dict["expired"] = expired
        if expires_at is not UNSET:
            field_dict["expiresAt"] = expires_at
        if token is not UNSET:
            field_dict["token"] = token

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        if not src_dict:
            return None
        d = src_dict.copy()
        expired = d.pop("expired", UNSET)

        expires_at = d.pop("expiresAt", UNSET)

        token = d.pop("token", UNSET)

        preview_token_spec = cls(
            expired=expired,
            expires_at=expires_at,
            token=token,
        )

        preview_token_spec.additional_properties = d
        return preview_token_spec

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.integration_additional_infos import IntegrationAdditionalInfos
    from ..models.integration_endpoints import IntegrationEndpoints
    from ..models.integration_headers import IntegrationHeaders
    from ..models.integration_organization import IntegrationOrganization
    from ..models.integration_query_params import IntegrationQueryParams
    from ..models.integration_repository import IntegrationRepository


T = TypeVar("T", bound="Integration")


@_attrs_define
class Integration:
    """Integration

    Attributes:
        additional_infos (Union[Unset, IntegrationAdditionalInfos]): Integration additional infos
        endpoints (Union[Unset, IntegrationEndpoints]): Integration endpoints
        headers (Union[Unset, IntegrationHeaders]): Integration headers
        name (Union[Unset, str]): Integration name
        organizations (Union[Unset, list['IntegrationOrganization']]): Integration organizations
        params (Union[Unset, IntegrationQueryParams]): Integration query params
        repositories (Union[Unset, list['IntegrationRepository']]): Integration repositories
    """

    additional_infos: Union[Unset, "IntegrationAdditionalInfos"] = UNSET
    endpoints: Union[Unset, "IntegrationEndpoints"] = UNSET
    headers: Union[Unset, "IntegrationHeaders"] = UNSET
    name: Union[Unset, str] = UNSET
    organizations: Union[Unset, list["IntegrationOrganization"]] = UNSET
    params: Union[Unset, "IntegrationQueryParams"] = UNSET
    repositories: Union[Unset, list["IntegrationRepository"]] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        additional_infos: Union[Unset, dict[str, Any]] = UNSET
        if (
            self.additional_infos
            and not isinstance(self.additional_infos, Unset)
            and not isinstance(self.additional_infos, dict)
        ):
            additional_infos = self.additional_infos.to_dict()
        elif self.additional_infos and isinstance(self.additional_infos, dict):
            additional_infos = self.additional_infos

        endpoints: Union[Unset, dict[str, Any]] = UNSET
        if self.endpoints and not isinstance(self.endpoints, Unset) and not isinstance(self.endpoints, dict):
            endpoints = self.endpoints.to_dict()
        elif self.endpoints and isinstance(self.endpoints, dict):
            endpoints = self.endpoints

        headers: Union[Unset, dict[str, Any]] = UNSET
        if self.headers and not isinstance(self.headers, Unset) and not isinstance(self.headers, dict):
            headers = self.headers.to_dict()
        elif self.headers and isinstance(self.headers, dict):
            headers = self.headers

        name = self.name

        organizations: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.organizations, Unset):
            organizations = []
            for organizations_item_data in self.organizations:
                if type(organizations_item_data) == dict:
                    organizations_item = organizations_item_data
                else:
                    organizations_item = organizations_item_data.to_dict()
                organizations.append(organizations_item)

        params: Union[Unset, dict[str, Any]] = UNSET
        if self.params and not isinstance(self.params, Unset) and not isinstance(self.params, dict):
            params = self.params.to_dict()
        elif self.params and isinstance(self.params, dict):
            params = self.params

        repositories: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.repositories, Unset):
            repositories = []
            for repositories_item_data in self.repositories:
                if type(repositories_item_data) == dict:
                    repositories_item = repositories_item_data
                else:
                    repositories_item = repositories_item_data.to_dict()
                repositories.append(repositories_item)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if additional_infos is not UNSET:
            field_dict["additionalInfos"] = additional_infos
        if endpoints is not UNSET:
            field_dict["endpoints"] = endpoints
        if headers is not UNSET:
            field_dict["headers"] = headers
        if name is not UNSET:
            field_dict["name"] = name
        if organizations is not UNSET:
            field_dict["organizations"] = organizations
        if params is not UNSET:
            field_dict["params"] = params
        if repositories is not UNSET:
            field_dict["repositories"] = repositories

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        from ..models.integration_additional_infos import IntegrationAdditionalInfos
        from ..models.integration_endpoints import IntegrationEndpoints
        from ..models.integration_headers import IntegrationHeaders
        from ..models.integration_organization import IntegrationOrganization
        from ..models.integration_query_params import IntegrationQueryParams
        from ..models.integration_repository import IntegrationRepository

        if not src_dict:
            return None
        d = src_dict.copy()
        _additional_infos = d.pop("additionalInfos", UNSET)
        additional_infos: Union[Unset, IntegrationAdditionalInfos]
        if isinstance(_additional_infos, Unset):
            additional_infos = UNSET
        else:
            additional_infos = IntegrationAdditionalInfos.from_dict(_additional_infos)

        _endpoints = d.pop("endpoints", UNSET)
        endpoints: Union[Unset, IntegrationEndpoints]
        if isinstance(_endpoints, Unset):
            endpoints = UNSET
        else:
            endpoints = IntegrationEndpoints.from_dict(_endpoints)

        _headers = d.pop("headers", UNSET)
        headers: Union[Unset, IntegrationHeaders]
        if isinstance(_headers, Unset):
            headers = UNSET
        else:
            headers = IntegrationHeaders.from_dict(_headers)

        name = d.pop("name", UNSET)

        organizations = []
        _organizations = d.pop("organizations", UNSET)
        for organizations_item_data in _organizations or []:
            organizations_item = IntegrationOrganization.from_dict(organizations_item_data)

            organizations.append(organizations_item)

        _params = d.pop("params", UNSET)
        params: Union[Unset, IntegrationQueryParams]
        if isinstance(_params, Unset):
            params = UNSET
        else:
            params = IntegrationQueryParams.from_dict(_params)

        repositories = []
        _repositories = d.pop("repositories", UNSET)
        for repositories_item_data in _repositories or []:
            repositories_item = IntegrationRepository.from_dict(repositories_item_data)

            repositories.append(repositories_item)

        integration = cls(
            additional_infos=additional_infos,
            endpoints=endpoints,
            headers=headers,
            name=name,
            organizations=organizations,
            params=params,
            repositories=repositories,
        )

        integration.additional_properties = d
        return integration

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import logging
import os

from blaxel.client.models import Metadata, Port, Runtime, Sandbox, SandboxSpec
from blaxel.sandbox.sandbox import SandboxInstance

logger = logging.getLogger(__name__)

async def local_sandbox(sandbox_name: str):
    os.environ[f"BL_SANDBOX_{sandbox_name.replace('-', '_').upper()}_URL"] = "http://localhost:8080"
    sandbox = SandboxInstance(Sandbox(
        metadata=Metadata(name=sandbox_name),
    ))
    return sandbox

async def create_or_get_sandbox(sandbox_name: str):
    # Create sandbox
    # return await local_sandbox(sandbox_name)
    try:
        sandbox = await SandboxInstance.get(sandbox_name)
        return sandbox
    except Exception:
        image = "blaxel/prod-base:latest"
        logger.info(f"Creating sandbox {sandbox_name} with image {image}")
        sandbox = await SandboxInstance.create(Sandbox(
            metadata=Metadata(name=sandbox_name),
            spec=SandboxSpec(
                runtime=Runtime(
                    image=image,
                    memory=2048,
                    cpu=2,
                    ports=[
                        Port(name="sandbox-api", target=8080, protocol="HTTP")
                    ]
                )
            )
        ))
        logger.info("Waiting for sandbox to be deployed")
        await sandbox.wait(max_wait=120000, interval=1000)
        logger.info("Sandbox deployed")
        return sandbox


import asyncio
import logging
import random
import string

from utils import create_or_get_sandbox

from blaxel.sandbox.client.models import ProcessRequest
from blaxel.sandbox.sandbox import SandboxInstance

logger = logging.getLogger(__name__)


def random_string(length: int):
    return ''.join(random.choices(string.ascii_letters + string.digits, k=length)).lower()


async def run_process_with_retry(sandbox: SandboxInstance, command: str):
    for _ in range(2):
        try:
            # new_client = client.with_base_url(sandbox.process.url).with_headers(settings.headers)
            # response = await new_client.get_async_httpx_client().get(f"/health")
            # print(response.text)
            dir = await sandbox.fs.ls("/")
            print(dir)
            process = await sandbox.process.exec(ProcessRequest(name="test", command=command))
            print(f"Process {process.name} started with pid {process.pid}")
            return process
        except Exception as e:
            print("Failed to run process, retrying...", e)
            await asyncio.sleep(1)

async def main():
    sandbox_name = random_string(10)
    try:
        print(f"Creating sandbox {sandbox_name}")
        sandbox = await create_or_get_sandbox(sandbox_name)
        sandbox = await SandboxInstance.get(sandbox_name)
        await run_process_with_retry(sandbox, 'pwd')
        await SandboxInstance.delete(sandbox_name)
    except Exception as e:
        print(e)
    finally:
        await SandboxInstance.delete(sandbox_name)

    sandbox_name = random_string(10)
    try:
        print(f"Creating sandbox {sandbox_name}")
        sandbox = await create_or_get_sandbox(sandbox_name)
        sandbox = await SandboxInstance.get(sandbox_name)
        await run_process_with_retry(sandbox, 'pwd')
        await SandboxInstance.delete(sandbox_name)
    except Exception as e:
        print(e)
    finally:
        await SandboxInstance.delete(sandbox_name)


if __name__ == "__main__":
    asyncio.run(main())



from typing import Any, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="JobsChartValue")


@_attrs_define
class JobsChartValue:
    """Jobs CPU usage

    Attributes:
        timestamp (Union[Unset, str]): Metric timestamp
        value (Union[Unset, int]): Metric value
    """

    timestamp: Union[Unset, str] = UNSET
    value: Union[Unset, int] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        timestamp = self.timestamp

        value = self.value

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if timestamp is not UNSET:
            field_dict["timestamp"] = timestamp
        if value is not UNSET:
            field_dict["value"] = value

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        if not src_dict:
            return None
        d = src_dict.copy()
        timestamp = d.pop("timestamp", UNSET)

        value = d.pop("value", UNSET)

        jobs_chart_value = cls(
            timestamp=timestamp,
            value=value,
        )

        jobs_chart_value.additional_properties = d
        return jobs_chart_value

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.knowledgebase_spec_options import KnowledgebaseSpecOptions
    from ..models.revision_configuration import RevisionConfiguration


T = TypeVar("T", bound="KnowledgebaseSpec")


@_attrs_define
class KnowledgebaseSpec:
    """Knowledgebase specification

    Attributes:
        collection_name (Union[Unset, str]): Collection name
        embedding_model (Union[Unset, str]): Embedding model
        embedding_model_type (Union[Unset, str]): Embedding model type
        enabled (Union[Unset, bool]): Enable or disable the agent
        integration_connections (Union[Unset, list[str]]):
        options (Union[Unset, KnowledgebaseSpecOptions]): Options specific to the knowledge base
        policies (Union[Unset, list[str]]):
        revision (Union[Unset, RevisionConfiguration]): Revision configuration
        sandbox (Union[Unset, bool]): Sandbox mode
    """

    collection_name: Union[Unset, str] = UNSET
    embedding_model: Union[Unset, str] = UNSET
    embedding_model_type: Union[Unset, str] = UNSET
    enabled: Union[Unset, bool] = UNSET
    integration_connections: Union[Unset, list[str]] = UNSET
    options: Union[Unset, "KnowledgebaseSpecOptions"] = UNSET
    policies: Union[Unset, list[str]] = UNSET
    revision: Union[Unset, "RevisionConfiguration"] = UNSET
    sandbox: Union[Unset, bool] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        collection_name = self.collection_name

        embedding_model = self.embedding_model

        embedding_model_type = self.embedding_model_type

        enabled = self.enabled

        integration_connections: Union[Unset, list[str]] = UNSET
        if not isinstance(self.integration_connections, Unset):
            integration_connections = self.integration_connections

        options: Union[Unset, dict[str, Any]] = UNSET
        if self.options and not isinstance(self.options, Unset) and not isinstance(self.options, dict):
            options = self.options.to_dict()
        elif self.options and isinstance(self.options, dict):
            options = self.options

        policies: Union[Unset, list[str]] = UNSET
        if not isinstance(self.policies, Unset):
            policies = self.policies

        revision: Union[Unset, dict[str, Any]] = UNSET
        if self.revision and not isinstance(self.revision, Unset) and not isinstance(self.revision, dict):
            revision = self.revision.to_dict()
        elif self.revision and isinstance(self.revision, dict):
            revision = self.revision

        sandbox = self.sandbox

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if collection_name is not UNSET:
            field_dict["collectionName"] = collection_name
        if embedding_model is not UNSET:
            field_dict["embeddingModel"] = embedding_model
        if embedding_model_type is not UNSET:
            field_dict["embeddingModelType"] = embedding_model_type
        if enabled is not UNSET:
            field_dict["enabled"] = enabled
        if integration_connections is not UNSET:
            field_dict["integrationConnections"] = integration_connections
        if options is not UNSET:
            field_dict["options"] = options
        if policies is not UNSET:
            field_dict["policies"] = policies
        if revision is not UNSET:
            field_dict["revision"] = revision
        if sandbox is not UNSET:
            field_dict["sandbox"] = sandbox

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        from ..models.knowledgebase_spec_options import KnowledgebaseSpecOptions
        from ..models.revision_configuration import RevisionConfiguration

        if not src_dict:
            return None
        d = src_dict.copy()
        collection_name = d.pop("collectionName", UNSET)

        embedding_model = d.pop("embeddingModel", UNSET)

        embedding_model_type = d.pop("embeddingModelType", UNSET)

        enabled = d.pop("enabled", UNSET)

        integration_connections = cast(list[str], d.pop("integrationConnections", UNSET))

        _options = d.pop("options", UNSET)
        options: Union[Unset, KnowledgebaseSpecOptions]
        if isinstance(_options, Unset):
            options = UNSET
        else:
            options = KnowledgebaseSpecOptions.from_dict(_options)

        policies = cast(list[str], d.pop("policies", UNSET))

        _revision = d.pop("revision", UNSET)
        revision: Union[Unset, RevisionConfiguration]
        if isinstance(_revision, Unset):
            revision = UNSET
        else:
            revision = RevisionConfiguration.from_dict(_revision)

        sandbox = d.pop("sandbox", UNSET)

        knowledgebase_spec = cls(
            collection_name=collection_name,
            embedding_model=embedding_model,
            embedding_model_type=embedding_model_type,
            enabled=enabled,
            integration_connections=integration_connections,
            options=options,
            policies=policies,
            revision=revision,
            sandbox=sandbox,
        )

        knowledgebase_spec.additional_properties = d
        return knowledgebase_spec

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import asyncio
import logging
import os
import traceback

from utils import create_or_get_sandbox

from blaxel.sandbox.base import ResponseError
from blaxel.sandbox.client.models import ProcessRequest

logger = logging.getLogger(__name__)


async def main():
    user = os.environ.get("USER")
    sandbox_name = "sandbox-test-3"
    try:
        sandbox = await create_or_get_sandbox(sandbox_name)

        # Filesystem tests
        await sandbox.fs.write(f"/Users/{user}/Downloads/test", "Hello world")
        content = await sandbox.fs.read(f"/Users/{user}/Downloads/test")
        assert content == "Hello world", "File content is not correct"
        dir = await sandbox.fs.ls(f"/Users/{user}/Downloads")
        assert dir.files and len(dir.files) >= 1, "Directory is empty"
        assert any(f.path == f"/Users/{user}/Downloads/test" for f in dir.files), "File not found in directory"
        await sandbox.fs.mkdir(f"/Users/{user}/Downloads/test2")
        after_mkdir = await sandbox.fs.ls(f"/Users/{user}/Downloads/test2")
        assert not after_mkdir.files or len(after_mkdir.files) == 0, "Directory is not empty after mkdir"
        await sandbox.fs.cp(f"/Users/{user}/Downloads/test", f"/Users/{user}/Downloads/test2/test")
        after_cp_ls = await sandbox.fs.ls(f"/Users/{user}/Downloads/test2")
        assert after_cp_ls.files and any(f.path == f"/Users/{user}/Downloads/test2/test" for f in after_cp_ls.files), "File not found in directory after cp"
        await sandbox.fs.rm(f"/Users/{user}/Downloads/test")
        try:
            await sandbox.fs.rm(f"/Users/{user}/Downloads/test2")
        except ResponseError as e:
            logger.info(f"That is expected => {e.error}")
        await sandbox.fs.rm(f"/Users/{user}/Downloads/test2", True)

        # Process tests
        process = await sandbox.process.exec(ProcessRequest(name="test", command="echo 'Hello world'"))
        assert getattr(process, "status", None) != "completed", "Process did complete without waiting"
        await asyncio.sleep(0.01)
        completed_process = await sandbox.process.get("test")
        assert getattr(completed_process, "status", None) == "completed", "Process did not complete"
        logs = await sandbox.process.logs("test")
        assert logs == 'Hello world\n', "Logs are not correct"
        try:
            await sandbox.process.kill("test")
        except ResponseError as e:
            logger.info(f"That is expected => {e.error}")
    except Exception as e:
        logger.error(f"Error => {e}")
        tb_str = traceback.format_exception(type(e), e, e.__traceback__)
        logger.error(f"Stacktrace: {''.join(tb_str)}")
    finally:
        logger.info("Deleting sandbox")
        # await SandboxInstance.delete(sandbox_name)

if __name__ == "__main__":
    asyncio.run(main())



import nest_asyncio

nest_asyncio.apply()

import asyncio

from dotenv import load_dotenv
from pydantic_ai import Agent, CallToolsNode
from pydantic_ai.messages import ToolCallPart

load_dotenv()

from logging import getLogger

from blaxel.tools import bl_tools

logger = getLogger(__name__)

async def test_mcp_tools_langchain():
    tools = await bl_tools(["blaxel-search"]).to_langchain()
    if len(tools) == 0:
        raise Exception("No tools found")
    result = await tools[0].ainvoke({ "query": "What is the capital of France?"})
    result = await tools[0].ainvoke({ "query": "What is the capital of USA?"})
    logger.info(result)

async def test_mcp_tools_llamaindex():
    tools = await bl_tools(["blaxel-search"]).to_llamaindex()
    if len(tools) == 0:
        raise Exception("No tools found")
    result = await tools[0].acall(query="What is the capital of France?")
    logger.info(result)

async def test_mcp_tools_crewai():
    tools = await bl_tools(["blaxel-search"]).to_crewai()
    if len(tools) == 0:
        raise Exception("No tools found")
    result = tools[0].run(query="What is the capital of France?")
    logger.info(result)

async def test_mcp_tools_pydantic():
    tools = await bl_tools(["blaxel-search"]).to_pydantic()
    if len(tools) == 0:
        raise Exception("No tools found")

    agent = Agent(model='openai:gpt-4o-mini', tools=tools)
    # result = await agent.run('What is the capital of France?', deps='human')
    async with agent.iter('Search what is the capital of France?') as agent_run:
        async for node in agent_run:
            # Each node represents a step in the agent's execution
            if isinstance(node, CallToolsNode):
                for part in node.model_response.parts:
                    if isinstance(part, ToolCallPart):
                        logger.info(f"Tool call: {part}")
                    else:
                        logger.info(f"Response: {part}")

async def test_mcp_tools_google_adk():
    tools = await bl_tools(["blaxel-search"]).to_google_adk()
    if len(tools) == 0:
        raise Exception("No tools found")
    result = await tools[0].run_async(args={"query": "What is the capital of France?"}, tool_context=None)
    logger.info(result)

async def test_mcp_tools_blaxel():
    tools = bl_tools(["blaxel-search"], {"test": "test"})
    await tools.intialize()
    blaxel_tools = tools.get_tools()
    if len(blaxel_tools) == 0:
        raise Exception("No tools found")
    result = await blaxel_tools[0].coroutine(query="What is the capital of France?")
    logger.info(result)
    result = await blaxel_tools[0].coroutine(query="What is the capital of Germany?")
    logger.info(result)
    await asyncio.sleep(7)
    result = await blaxel_tools[0].coroutine(query="What is the capital of USA?")
    logger.info(result)
    result = await blaxel_tools[0].coroutine(query="What is the capital of Canada?")
    logger.info(result)

async def main():
    await test_mcp_tools_blaxel()
    await test_mcp_tools_langchain()
    await test_mcp_tools_llamaindex()
    await test_mcp_tools_crewai()
    await test_mcp_tools_pydantic()
    await test_mcp_tools_google_adk()

if __name__ == "__main__":
    asyncio.run(main())